@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -57,6 +60,14 @@ class CloudFunctionsServiceAsyncClient:
     parse_cloud_function_path = staticmethod(
         CloudFunctionsServiceClient.parse_cloud_function_path
     )
+    crypto_key_path = staticmethod(CloudFunctionsServiceClient.crypto_key_path)
+    parse_crypto_key_path = staticmethod(
+        CloudFunctionsServiceClient.parse_crypto_key_path
+    )
+    repository_path = staticmethod(CloudFunctionsServiceClient.repository_path)
+    parse_repository_path = staticmethod(
+        CloudFunctionsServiceClient.parse_repository_path
+    )
     common_billing_account_path = staticmethod(
         CloudFunctionsServiceClient.common_billing_account_path
     )
@@ -117,6 +128,42 @@ class CloudFunctionsServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return CloudFunctionsServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> CloudFunctionsServiceTransport:
         """Returns the transport used by the client instance.
@@ -281,6 +328,7 @@ class CloudFunctionsServiceAsyncClient:
                 contains user computation executed in
                 response to an event. It encapsulate
                 function and triggers configurations.
+                Next tag: 36
 
         """
         # Create or coerce a protobuf request object.
@@ -373,7 +421,7 @@ class CloudFunctionsServiceAsyncClient:
 
                 The result type for the operation will be :class:`google.cloud.functions_v1.types.CloudFunction` Describes a Cloud Function that contains user computation executed in
                    response to an event. It encapsulate function and
-                   triggers configurations.
+                   triggers configurations. Next tag: 36
 
         """
         # Create or coerce a protobuf request object.
@@ -457,7 +505,7 @@ class CloudFunctionsServiceAsyncClient:
 
                 The result type for the operation will be :class:`google.cloud.functions_v1.types.CloudFunction` Describes a Cloud Function that contains user computation executed in
                    response to an event. It encapsulate function and
-                   triggers configurations.
+                   triggers configurations. Next tag: 36
 
         """
         # Create or coerce a protobuf request object.
